/*
 * $Id: Square.java,v 1.3 1998/04/05 20:13:59 lozac Exp $
 *
 *
 * $Log: Square.java,v $
 * Revision 1.3  1998/04/05 20:13:59  lozac
 * Added the GUI getComponent() common code.
 *
 * Revision 1.2  1998/04/05 18:44:23  vmallet
 * Added the Serializable implementation.
 * Added the getComponent() prototype.
 *
 * Revision 1.1  1998/04/04 14:32:06  lozac
 * Initial revision
 *
 *
 */ 

package scrobble.square;

import scrobble.*;
import game.*;


import java.awt.*; 
import java.io.*;
import java.util.*;
import com.sun.java.swing.*;
import com.sun.java.swing.border.*;

/**
 * Square.java
 * 
 * The <code>Square</code> class represents a square in the <code>Board</code> of Scrabble.
 * It contains <code>Letter</code> objects. It is the root class of a wide hierarchy of other
 * squares class.
 *
 * @see Board
 * @see Letter
 * @version $Revision: 1.3 $
 * @author  Frederique Poivert (poivert@enst.fr)
 */
public class Square implements Serializable { 


  /**
   * Letter that is contained by the Square
   */
  transient Letter letter;

  /**
   * The identity of the last Square created. 
   */
  static int generalId;

  /** 
   * the identity of this square.
   */
  int id;

  /**
   *
   */
  String iconName = "images/fond-clair.gif";

  /**
   * Square must be Observable by the Square of the Client. 
   * So needs the object o to have the methods addObserver(), deleteObserver() 
   * and notifyObserver().
   */
  transient protected Observable o;

   /**
   * The request which indicates that the Square becomes empty.
   */
  public static final int RQ_SQUAREEMPTY = 7010;

   /**
   * The request which indicates that the Square received a letter.
   */
  public static final int RQ_SQUAREGETLETTER = 7020;
  
  /** 
   * Frame group : for communication between Square objects 
   */    
  public static final int SQUARE_GROUP   = 1070;

  /**
   * The Panel which represents the DoubleLetterSquare.
   */
  JLabel squareLabel;

  // Constructor


  /**
   * Creates a new Square
   */
  public Square() { 
    letter = null;
    o = new Observable() {
      public void notifyObservers(Object o) {
	setChanged();
	super.notifyObservers(o);
      }
    };
  }


  // Methods

  /**
   * Creates a new LinkFrame, which concerns only one Square. 
   * @param request the request of the frame.
   * @param data the data of the frame (The Letter in the Square of nothing if the Square becomes Empty).
   * @param squareid the identity of the Square which is concerned by the request.
   */  
  public LinkFrame createLinkframe(int request, Object data, int squareid) {
    LinkFrame frame = new LinkFrame();
    
    frame.setGroup(SQUARE_GROUP);
    frame.setRequest(request, squareid);
    frame.setData(data);

    return frame;  
  }

  /**
   * Invoked when a new data frame is received.
   * @param frame the frame that has just been received.
   */  
  public void frameReceived(Object link, LinkFrame frame) {
    int request = frame.getRequest();
    
    if ( frame.getSubRequest() == id) {
      switch (request) {
      case RQ_SQUAREEMPTY :
	setEmpty(); 
	break;
      case RQ_SQUAREGETLETTER :  
	if (isEmpty() == true) {
	    putLetter((Letter) frame.getData());
	}
	else throw new InternalError("Square already full !");
	break;
      default : 
	System.out.println("Unhandled request");  
	break;
      }
    }
  }

  /**
   * Adds an observer to the set of observers for this object.
   * @param obs an observer to be added.
   */
  public void addObserver(Observer obs) {
    o.addObserver(obs);
  }

  /**
   * Deletes an observer to the set of observers for this object.
   * @param obs the observer to be deleted
   */
  public void deleteObserver(Observer obs) {
    o.deleteObserver(obs);
  }
  
  /**
   * Clears the observerlist so that this object no longer has any observers.
   */
  public void deleteObservers() {
    o.deleteObservers();
  }
  
  /**
   * Notifies all the observers of this Square that it has changed and indicates this 
   * changement by a frame.
   * @param frame the Linkframe that indicates the changement.
   */
  public void notifyObservers(LinkFrame frame) {
    o.notifyObservers(frame);
  }

 /**
   * Returns the number of observers of this object.
   * @return the number of observers of this object.
   */  
  public int countObservers() {
    return o.countObservers();
  }

    /**
   * This method is called when a Square is created. 
   * It gives to this new Square his identity.
   * @return the Square's new identity;
   */  

  public synchronized int getNewId() {
    int newId;

    newId = generalId;
    generalId++;
    return newId;
  } 

  /**
   * Return the Square's identity 
   * @return the Square's identity 
   */
  public int getId() {
    return id;
  }

  /**
   * Puts a letter in the Square
   * @param newletter the new letter contained by the Square
   */
  public void putLetter(Letter newletter) {
    LinkFrame frame;

    letter = newletter;
    frame = createLinkframe(RQ_SQUAREGETLETTER, letter, getId());
    notifyObservers(frame);
  }

  /**
   * Returns the letter that is contained by the Square
   * @return the letter
   */  
  public Letter getLetter() {
    return letter;
  }


  /**
   * Empties the Square
   */
  public void setEmpty() {
    LinkFrame frame;

    letter = null;
    frame = createLinkframe(RQ_SQUAREEMPTY, null, getId());
    notifyObservers(frame);
  }

  /**
   * Indicates if a Square is empty or not
   * @return true if the Square is empty, false otherwise
   */  
  public boolean isEmpty() {
    return getLetter() == null;
  }
  
  /**
   * Returns a string representation of the object.
   */
  public String toString() {     
    String name = getClass().getName();
    if (isEmpty() == true)
      return name.substring(name.lastIndexOf(".") + 1) + " (vide) ";
    else
      return name.substring(name.lastIndexOf(".") + 1) + " (" + getLetter() + ") ";
	
  }


  /**
   * Returns a GUI representation of the Square.
   */
  public Component getComponent() {
    checkComponent(iconName);
    return squareLabel;
  }

    /**
   * Helper for the representation of the Square. 
   */
  public void checkComponent(String iconName) {
    if (squareLabel ==  null) {
      squareLabel = new JLabel();
      if (isEmpty()) {
	Icon squareIcon = new ImageIcon(iconName); 
	squareLabel.setIcon(squareIcon);
      }
      else {
	squareLabel.add(getLetter().getComponent());
      }
    }
    
  }
  
}

